/*
 * iReport - Visual Designer for JasperReports.
 * Copyright (C) 2002 - 2009 Jaspersoft Corporation. All rights reserved.
 * http://www.jaspersoft.com
 *
 * Unless you have purchased a commercial license agreement from Jaspersoft,
 * the following license terms apply:
 *
 * This program is part of iReport.
 *
 * iReport is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * iReport is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with iReport. If not, see <http://www.gnu.org/licenses/>.
 */
package com.jaspersoft.ireport.designer.menu.preview;

import com.jaspersoft.ireport.designer.IReportManager;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;

/**
 *
 * @author gtoffoli
 */
public abstract class AbstractPreviewAction extends CallableSystemAction implements PreferenceChangeListener, ItemListener {

    public abstract String getPreviewType();

    private JCheckBoxMenuItem item;
    
    public AbstractPreviewAction()
    {
        item = new JCheckBoxMenuItem(getName());
        IReportManager.getPreferences().addPreferenceChangeListener(this);
        preferenceChange(null);
        item.addItemListener(this);
    }
    
    
    @Override
    public JMenuItem getMenuPresenter() {
        
        return item;
    }
     
    
    public void performAction() {
        
        if (item.isSelected())
        {
            if (getPreviewType().length() > 0)
            {
                IReportManager.getPreferences().put("output_format", getPreviewType());
            }
            else
            {
                IReportManager.getPreferences().remove("output_format");
            }
        }
    }

    @Override
    protected void initialize() {
        super.initialize();
        
        // see org.openide.util.actions.SystemAction.iconResource() Javadoc for more details
        putValue("noIconInMenu", Boolean.TRUE);
        
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    protected boolean asynchronous() {
        return false;
    }
    
    public void preferenceChange(PreferenceChangeEvent evt)
    {
        item.setSelected(  IReportManager.getPreferences().get("output_format", "").equals(getPreviewType()) ); 
    }
    
    
    public void itemStateChanged(ItemEvent e)
    {
        performAction();
    }

}
